package com.io.yutian.enderdragonwar.game;

import com.io.yutian.enderdragonwar.EnderDragonWar;
import com.io.yutian.enderdragonwar.util.BossBarUtil;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.GameRule;
import org.bukkit.World;
import org.bukkit.attribute.Attribute;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

public class Game {

    private Player player;
    private BossBar bossBar1;
    private BossBar bossBar2;
    private boolean startd;
    private long liveTime;
    private List<BukkitTask> tasks = new ArrayList<>();
    private int tragetCueI = 0;

    private int dianzan;

    public Game() {
        bossBar1 = Bukkit.createBossBar("下播时间", BarColor.WHITE, BarStyle.SEGMENTED_20);
        bossBar2 = Bukkit.createBossBar("挑战目标", BarColor.WHITE, BarStyle.SOLID);
    }

    public void updateBossBar() {
        if (player == null) {
            return;
        }
        // 击败末影龙提示
        targetCueBossBar();
        // 获取最大时间
        long maxLiveTime = 1000 * 60 * 60 * 6;
        double d = (double) getLiveTime() / (double) maxLiveTime;
        d = Math.max(0, Math.min(d, 1));
        long liveTimeInSeconds = getLiveTime() / 1000;
        long hours = liveTimeInSeconds / 3600;
        long minutes = (liveTimeInSeconds % 3600) / 60;
        long seconds = liveTimeInSeconds % 60;
        if (hours >= 1) {
            bossBar1.setTitle("§6§l距离下播还有§f§l: §a" + hours + "小时" + minutes + "分" + seconds + "秒");
        } else if (minutes >= 1) {
            bossBar1.setTitle("§6§l距离下播还有§f§l: §a" + minutes + "分" + seconds + "秒");
        } else if (seconds >= 1) {
            bossBar1.setTitle("§6§l距离下播还有§f§l: §a" + seconds + "秒");
        } else {
            bossBar1.setTitle("§a§l恭喜您完成挑战！下播吧！");
        }
        BossBarUtil.setBarProgress(bossBar1, d);
        BossBarUtil.setBarColor(bossBar1, d);
    }

    public void targetCueBossBar() {
        tragetCueI--;
        if (tragetCueI <= 0) {
            bossBar2.addPlayer(player);
            List<String> stringList = new ArrayList<>();
            stringList.add("§c§l击败末影龙  §b§l减播1小时！");
            Collections.shuffle(stringList);
            Random random = new Random();
            String titleShow = stringList.get(random.nextInt(stringList.size()));
            bossBar2.setTitle(titleShow);
            BossBarUtil.setBarColor(bossBar2);
            tragetCueI = 20;
        } else {
            if (tragetCueI == 16) {
                if (bossBar2.getPlayers().contains(player)) {
                    bossBar2.removePlayer(player);
                }
            }
        }
    }

    public void startGame(Player player) {
        if (startd) {
            rejoinGame(player);
            return;
        }
        this.player = player;
        World world = this.player.getWorld();
        world.setTime(1000);
        world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        world.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
        this.startd = true;
        this.liveTime = 1000 * 60 * 60 * 3;
        initPlayerData(player);
        tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                liveTime = liveTime - 500;
                updateBossBar();
            }
        }.runTaskTimer(EnderDragonWar.inst(), 0L, 10L));
    }

    public void rejoinGame(Player player) {
        this.player = player;
        initPlayerData(player);
    }

    public void quitGame(Player player) {
        this.player = null;
    }

    public void stop() {
        this.player = null;
        this.startd = false;
        tasks.forEach(BukkitTask::cancel);
        removeBossBar();
    }

    public Player getPlayer() {
        return player;
    }

    public World getWorld() {
        return player.getWorld();
    }

    public void initPlayerData(Player player) {
        player.setFoodLevel(20);
        player.setGameMode(GameMode.SURVIVAL);
        Attribute[] attributes = new Attribute[]{Attribute.GENERIC_MAX_HEALTH, Attribute.GENERIC_MOVEMENT_SPEED, Attribute.GENERIC_SCALE, Attribute.GENERIC_STEP_HEIGHT};
        double[] attributeValues = new double[]{20, 0.1, 1, 0.6};
        for (int i = 0; i < attributes.length; i++) {
            player.getAttribute(attributes[i]).setBaseValue(attributeValues[i]);
        }
        bossBar1.addPlayer(player);
        bossBar2.addPlayer(player);
    }

    public void removeBossBar() {
        bossBar1.removeAll();
        bossBar2.removeAll();
        bossBar1 = Bukkit.createBossBar("下播时间", BarColor.WHITE, BarStyle.SEGMENTED_20);
        bossBar2 = Bukkit.createBossBar("挑战目标", BarColor.WHITE, BarStyle.SOLID);
    }

    public boolean isStartd() {
        return startd;
    }

    public void addTasks(BukkitTask task) {
        this.tasks.add(task);
    }

    public void addDianZan(int count) {
        this.dianzan += count;
    }

    public int getDianZan() {
        return dianzan;
    }

    public void setDianZan(int dianzan) {
        this.dianzan = dianzan;
    }

    public long getLiveTime() {
        return liveTime;
    }

    public void setLiveTime(long liveTime) {
        this.liveTime = liveTime;
    }

    public void takeLiveTime(long minutes) {
        long minutesTime = 1000 * 60 * minutes;
        long newTime = getLiveTime() - minutesTime;
        if (newTime < 0) {
            newTime = 0;
        }
        setLiveTime(newTime);
    }

    public void addLiveTime(long minutes) {
        long minutesTime = 1000 * 60 * minutes;
        long newTime = getLiveTime() + minutesTime;
        if (newTime < 0) {
            newTime = 0;
        }
        setLiveTime(newTime);
    }


}
