package com.io.yutian.enderdragonwar.listener;

import com.io.yutian.enderdragonwar.effect.types.LuckyBlockEffect;
import com.io.yutian.enderdragonwar.effect.types.SpawnEnderPortalEffect;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.entity.*;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.player.PlayerChangedWorldEvent;
import org.bukkit.event.player.PlayerInteractEntityEvent;
import org.bukkit.event.player.PlayerPortalEvent;
import org.bukkit.event.player.PlayerRespawnEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;

import java.util.HashMap;
import java.util.Map;

public class PlayerListener implements Listener {

    private static Map<Player, ItemStack> totem = new HashMap<>();

    @EventHandler
    public void onEntityDamage(EntityDamageEvent event) {
        if (event.getEntity() instanceof Player player) {
            PlayerInventory inventory = player.getInventory();
            Material item = Material.TOTEM_OF_UNDYING;
            ItemStack mainhand = inventory.getItemInMainHand();
            ItemStack offhand = inventory.getItemInOffHand();
            if (player.getHealth() - event.getFinalDamage() <= 0.0d && inventory.contains(item)) {
                if (mainhand != null && mainhand.getType() == item) {
                    return;
                }
                if ((offhand == null || offhand.getType() != item)) {
                    this.totem.put(player, inventory.getItemInOffHand());
                    ItemStack[] contents = inventory.getContents();
                    int length = contents.length;
                    int i = 0;
                    while (true) {
                        if (i >= length) {
                            break;
                        }
                        ItemStack totem = contents[i];
                        if (totem != null && totem.getType().equals(item)) {
                            totem.setAmount(totem.getAmount() - 1);
                            break;
                        }
                        i++;
                    }
                    inventory.setItemInOffHand(new ItemStack(Material.TOTEM_OF_UNDYING));
                }
            }
        }
    }

    @EventHandler
    public void onPlayerBreakBlock(BlockBreakEvent event) {
        Player player = event.getPlayer();
        Block block = event.getBlock();
        if (block.getType() == Material.SPONGE) {
            event.setCancelled(true);
            block.setType(Material.AIR);
            LuckyBlockEffect.openLuckyBlock(player, block);
        }
    }

    @EventHandler
    public void onPlayerRespawn(PlayerRespawnEvent event) {
        Player player = event.getPlayer();
        Location location = event.getRespawnLocation();
        for (int i = -3; i <= 3; i++) {
            for (int j = 0; j <= 3; j++) {
                for (int z = -3; z <= 3; z++) {
                    Block block = location.clone().add(i, j, z).getBlock();
                    if (block.getType() == Material.LAVA) {
                        block.setType(Material.AIR);
                    }
                }
            }
        }
        for (int i = -2; i <= 2; i++) {
            for (int j = -2; j <= 2; j++) {
                Block block = location.clone().add(i, -1, j).getBlock();
                block.setType(Material.BEDROCK);
            }
        }
    }

    @EventHandler
    public void onPlayerPortal(PlayerPortalEvent event) {
        Player player = event.getPlayer();
        Location location = player.getLocation();
        SpawnEnderPortalEffect.checkPortalRegion(location);
    }

    @EventHandler
    public void onPlayerChangeWorld(PlayerChangedWorldEvent event) {
        World from = event.getFrom();
        if (from.getEnvironment() == World.Environment.THE_END) {
            if (from.getEnderDragonBattle().hasBeenPreviouslyKilled()) {
                Bukkit.unloadWorld(from, false);
                Bukkit.createWorld(new WorldCreator(from.getName()).environment(World.Environment.THE_END));
            }
        }
    }


    @EventHandler
    public void onPlayerInteractEntity(PlayerInteractEntityEvent event) {
        Player player = event.getPlayer();
        Entity entity = event.getRightClicked();
        if (entity instanceof IronGolem) {
            entity.addPassenger(player);
        }
    }


}
